unit uVCMTDLL;

{$IFDEF FPC}
  {$mode objfpc}{$H+}
{$ENDIF}

interface

uses
  {$IFDEF FPC}
    dynlibs,
  {$ELSE}
    windows,
  {$ENDIF}
  SysUtils;

const
  {$IFNDEF FPC}
    SharedSuffix = 'dll';
  {$ENDIF}

  {$IFDEF VCMTDLL_DEBUG}
    vcmtdlldllname = 'dvcmtdll.' + SharedSuffix;
  {$ELSE}
    vcmtdlldllname = 'vcmtdll.' + SharedSuffix;
  {$ENDIF}

type
  EDLLLoadError   = class(Exception);
  EDLLImportError = class(Exception)
  public
    WrongFunc : String;
  end;

  MT_THREAD_HANDLE   = Pointer;
  Tvcmtdll_NewThread                   = function : MT_THREAD_HANDLE; cdecl;
  Tvcmtdll_DeleteThread                = procedure(ahandle : MT_THREAD_HANDLE); cdecl;
  Tvcmtdll_RegisterCallback            = procedure(ahandle : MT_THREAD_HANDLE; pCBFunc : Pointer; pCBData : Pointer); cdecl;
  Tvcmtdll_StartThread                 = procedure(ahandle : MT_THREAD_HANDLE; iDelay : Cardinal); cdecl;


var
  vcmtdll_NewThread          : Tvcmtdll_NewThread      = nil;
  vcmtdll_DeleteThread       : Tvcmtdll_DeleteThread   = nil;
  vcmtdll_RegisterCallback   : Tvcmtdll_RegisterCallback = nil;
  vcmtdll_StartThread        : Tvcmtdll_StartThread      = nil;


{handling the library}
function LoadDll:   Boolean;
function UnloadDll: Boolean;

var
  vcmtdll_dllHandle    : THandle;
  vcmtdllLoaded        : Boolean;
  vcmtdllUseExceptions : Boolean;

implementation

function UnloadDll: Boolean;
begin
  if vcmtdll_dllHandle <> 0 then
  begin
    {$IFDEF FPC}
    Result := UnloadLibrary(vcmtdll_dllHandle);
    {$ELSE}
    Result := FreeLibrary(vcmtdll_dllHandle);
    {$ENDIF}
    vcmtdll_dllHandle := 0;
  end
  else Result := true;
end;

function LoadDll(adllname : string): Boolean;
begin
  UnloadDll;
  vcmtdll_dllHandle := LoadLibrary(PCHAR(adllname));
  Result := (vcmtdll_dllHandle <> 0);
  if vcmtdllUseExceptions and not Result then
  begin
    raise Exception.Create('cannot load ' + adllname);
  end;
end;

function ImportDllFunction(dllhandle : THandle; funcname : string): Pointer;
var
  E : EDLLImportError;
begin
  Result := GetProcAddress(dllhandle, PChar(funcname));
  if Result = nil then
  begin
    E := EDllImportError.CreateFmt('could not find dll function "%s"', [funcname]);
    E.WrongFunc := funcname;
    raise E;
  end;
end;

function LoadFunctions(dllhandle : THandle): Boolean;
begin
  Result := false;
  try
    vcmtdll_NewThread                   := Tvcmtdll_NewThread(ImportDllFunction(dllhandle,'vcmtdll_NewThread'));
    vcmtdll_DeleteThread                := Tvcmtdll_DeleteThread(ImportDllFunction(dllhandle,'vcmtdll_DeleteThread'));
    vcmtdll_RegisterCallback            := Tvcmtdll_RegisterCallback(ImportDllFunction(dllhandle,'vcmtdll_RegisterCallback'));
    vcmtdll_StartThread                 := Tvcmtdll_StartThread(ImportDllFunction(dllhandle,'vcmtdll_StartThread'));

    Result := true;
  except
    on E: EDLLImportError do
    begin
      if vcmtdllUseExceptions then raise;
    end;
  end;
end;

procedure UnloadFunctions;
begin
  vcmtdll_NewThread                   := nil;
  vcmtdll_DeleteThread                := nil;
  vcmtdll_RegisterCallback            := nil;
  vcmtdll_StartThread                 := nil;

end;

function LoadDll: Boolean;
begin
  if not vcmtdllLoaded then
  begin
    if LoadDll(vcmtdlldllname) then
    begin
      if LoadFunctions(vcmtdll_dllHandle) then
      begin
        vcmtdllLoaded := true;
      end;
    end;
  end;
  Result := vcmtdllLoaded;
end;

function UnloadSidex: Boolean;
begin
  if vcmtdllLoaded then
  begin
    UnloadFunctions;
    Result := UnloadDll();
    vcmtdllLoaded := false;
  end
  else Result := true;
end;

initialization

vcmtdll_dllHandle    := 0;
vcmtdllLoaded        := false;
vcmtdllUseExceptions := true;

end.

